import MaxFlow.MaxFlow;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.implementations.SingleGraph;
import org.graphstream.stream.GraphParseException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class Autoroute
{

	public static void main(String[] args) throws IOException, GraphParseException {
		Graph g = new SingleGraph("Autoroute");
		g.read("data/reseauRoutier.dgs");

		g.setAttribute("ui.quality");
		g.setAttribute(("ui.antialias"));
		g.setAttribute("ui.stylesheet", "url('data/style.css')");
		g.nodes().forEach(n -> n.setAttribute("label", n.getId()));

		Autoroute.flotOptimal(g);
		MaxFlow mf = Autoroute.maxFlot(g);

		System.setProperty("org.graphstream.ui", "swing");

		System.out.println(mf.getFlow());
		g.edges().forEach(e -> {
			double flow = mf.getFlow(e);
			double cap = mf.getCapacity(e);
			if (flow > 0) e.setAttribute("ui.label", "" + (int)flow+"/" + e.getAttribute("cap"));
			if (cap == flow) e.setAttribute("ui.style", "fill-color: red;");
		});
		g.display(false);

	}

	/**
	 * Méthode qui lance l'algorithme de flowmax
	 * @param g
	 * @return
	 */
	private static MaxFlow maxFlot(Graph g)
	{
		MaxFlow mf = new MaxFlow();
		mf.setCapacityAttribute("cap");
		mf.init(g);
		mf.setSource(g.getNode("A"));
		mf.setSink(g.getNode("I"));
		mf.compute();
		return mf;
	}

	/**
	 * Méthode qui donne les flots successifs
	 * @param g
	 */
	private static void flotOptimal(Graph g)
	{
		int arcCap;
		double flotMax = 0.0;
		Edge arcAugmenter = null;
		MaxFlow mf;

		//Permet de récuperer le flotMax théorique du graphe
		int flotMaxGraphe = 0;
		List<Edge> VoisinDepart = g.getNode(0).leavingEdges().collect(Collectors.toList());
		for (Edge e :VoisinDepart)
			flotMaxGraphe += (int) e.getAttribute("cap");


		ArrayList<Edge> arcParcourir = Autoroute.tronconACDHI(g);

		do
		{
			for (int i = 0; i<g.edges().collect(Collectors.toList()).size(); i++)
			{
				Edge e = g.edges().collect(Collectors.toList()).get(i);
				arcCap = (int) e.getAttribute("cap");
				e.setAttribute("cap", 2500);
				mf = Autoroute.maxFlot(g);

				if(flotMax <mf.getFlow())
				{
					arcAugmenter = e;
					flotMax = mf.getFlow();
				}
				e.setAttribute("cap", arcCap);
			}
			g.getEdge(arcAugmenter.getId()).setAttribute("cap", (int) arcAugmenter.getAttribute("cap")+2500 );
			System.out.println("Arc augmenté: [" + arcAugmenter.getSourceNode()+"->"+ arcAugmenter.getTargetNode()+ "] | flot max : " + flotMax);
		}while(flotMax!= flotMaxGraphe);

	}


	private static ArrayList<Edge> tronconACDHI(Graph g)
	{
		ArrayList<Edge> alEdge = new ArrayList<>();

		alEdge.add(g.getEdge("e5")); //CD
		alEdge.add(g.getEdge("e9")); //DH

		return alEdge;

	}

}
